@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2015 David R. MacIver
+# Most of this work is copyright (C) 2013-2017 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -19,13 +19,15 @@ from __future__ import division, print_function, absolute_import
 
 import os
 import re
-import sqlite3
 import binascii
 import threading
 from contextlib import contextmanager
 
-from hypothesis.internal.compat import sha1, b64decode, b64encode
+from hypothesis._settings import note_deprecation
+from hypothesis.internal.compat import FileNotFoundError, sha1, hbytes, \
+    b64decode, b64encode
 
+sqlite3 = None
 SQLITE_PATH = re.compile(r"\.\(db|sqlite|sqlite3\)$")
 
 
@@ -77,6 +79,21 @@ class ExampleDatabase(EDMeta('ExampleDatabase', (object,), {})):
         """
         raise NotImplementedError('%s.delete' % (type(self).__name__))
 
+    def move(self, src, dest, value):
+        """Move value from key src to key dest. Equivalent to delete(src,
+        value) followed by save(src, value) but may have a more efficient
+        implementation.
+
+        Note that value will be inserted at dest regardless of whether
+        it is currently present at src.
+
+        """
+        if src == dest:
+            self.save(src, value)
+            return
+        self.delete(src, value)
+        self.save(dest, value)
+
     def fetch(self, key):
         """Return all values matching this key."""
         raise NotImplementedError('%s.fetch' % (type(self).__name__))
@@ -99,10 +116,10 @@ class InMemoryExampleDatabase(ExampleDatabase):
             yield v
 
     def save(self, key, value):
-        self.data.setdefault(key, set()).add(value)
+        self.data.setdefault(key, set()).add(hbytes(value))
 
     def delete(self, key, value):
-        self.data.get(key, set()).discard(value)
+        self.data.get(key, set()).discard(hbytes(value))
 
     def close(self):
         pass
@@ -114,6 +131,22 @@ class SQLiteExampleDatabase(ExampleDatabase):
         self.path = path
         self.db_created = False
         self.current_connection = threading.local()
+        global sqlite3
+        import sqlite3
+
+        if path == u':memory:':
+            note_deprecation(
+                'The SQLite database backend has been deprecated. '
+                'Use InMemoryExampleDatabase or set database_file=":memory:" '
+                'instead.'
+            )
+        else:
+            note_deprecation(
+                'The SQLite database backend has been deprecated. '
+                'Set database_file to some path name not ending in .db, '
+                '.sqlite or .sqlite3 to get the new directory based database '
+                'backend instead.'
+            )
 
     def connection(self):
         if not hasattr(self.current_connection, 'connection'):
@@ -139,7 +172,7 @@ class SQLiteExampleDatabase(ExampleDatabase):
                 yield cursor
             finally:
                 cursor.close()
-        except:
+        except BaseException:
             conn.rollback()
             raise
         else:
@@ -234,13 +267,20 @@ class DirectoryBasedExampleDatabase(ExampleDatabase):
     def fetch(self, key):
         kp = self._key_path(key)
         for path in os.listdir(kp):
-            with open(os.path.join(kp, path), 'rb') as i:
-                yield i.read()
+            try:
+                with open(os.path.join(kp, path), 'rb') as i:
+                    yield hbytes(i.read())
+            except FileNotFoundError:
+                pass
 
     def save(self, key, value):
         path = self._value_path(key, value)
         if not os.path.exists(path):
-            tmpname = path + '.' + str(binascii.hexlify(os.urandom(16)))
+            suffix = binascii.hexlify(os.urandom(16))
+            if not isinstance(suffix, str):  # pragma: no branch
+                # On Python 3, binascii.hexlify returns bytes
+                suffix = suffix.decode('ascii')
+            tmpname = path + '.' + suffix
             with open(tmpname, 'wb') as o:
                 o.write(value)
             try:
@@ -249,6 +289,17 @@ class DirectoryBasedExampleDatabase(ExampleDatabase):
                 os.unlink(tmpname)
             assert not os.path.exists(tmpname)
 
+    def move(self, src, dest, value):
+        if src == dest:
+            self.save(src, value)
+            return
+        try:
+            os.rename(
+                self._value_path(src, value), self._value_path(dest, value))
+        except OSError:
+            self.delete(src, value)
+            self.save(dest, value)
+
     def delete(self, key, value):
         try:
             os.unlink(self._value_path(key, value))
