@@ -186,3 +186,26 @@ class Frozen(HypothesisException):
 
     """Raised when a mutation method has been called on a ConjectureData object
     after freeze() has been called."""
+
+
+class MultipleFailures(HypothesisException):
+    """Indicates that Hypothesis found more than one distinct bug when testing
+    your code."""
+
+
+class DeadlineExceeded(HypothesisException):
+    """Raised when an individual test body has taken too long to run."""
+
+    def __init__(self, runtime, deadline):
+        super(DeadlineExceeded, self).__init__((
+            'Test took %.2fms, which exceeds the deadline of '
+            '%.2fms') % (runtime, deadline))
+        self.runtime = runtime
+        self.deadline = deadline
+
+
+class StopTest(BaseException):
+
+    def __init__(self, testcounter):
+        super(StopTest, self).__init__(repr(testcounter))
+        self.testcounter = testcounter
