@@ -17,27 +17,30 @@
 
 from __future__ import division, print_function, absolute_import
 
+import enum
 import math
 import datetime as dt
 import operator
 from decimal import Context, Decimal
-from inspect import isclass
-from numbers import Rational
+from inspect import isclass, isfunction
 from fractions import Fraction
+from functools import reduce
 
 from hypothesis.errors import InvalidArgument, ResolutionFailed
 from hypothesis.control import assume
 from hypothesis._settings import note_deprecation
+from hypothesis.internal.cache import LRUReusedCache
 from hypothesis.searchstrategy import SearchStrategy
 from hypothesis.internal.compat import gcd, ceil, floor, hrange, \
-    text_type, integer_types, get_type_hints, getfullargspec, \
-    implements_iterator
+    text_type, get_type_hints, getfullargspec, implements_iterator
 from hypothesis.internal.floats import is_negative, float_to_int, \
     int_to_float, count_between_floats
-from hypothesis.internal.coverage import check_function
 from hypothesis.internal.renaming import renamed_arguments
 from hypothesis.utils.conventions import infer, not_set
 from hypothesis.internal.reflection import proxies, required_args
+from hypothesis.internal.validation import check_type, try_convert, \
+    check_strategy, check_valid_bound, check_valid_sizes, \
+    check_valid_integer, check_valid_interval
 
 __all__ = [
     'nothing',
@@ -85,9 +88,10 @@ def convert_value(v):
     return (type(v), v)
 
 
-def cacheable(fn):
-    cache = {}
+STRATEGY_CACHE = LRUReusedCache(1024)
 
+
+def cacheable(fn):
     @proxies(fn)
     def cached_strategy(*args, **kwargs):
         kwargs_cache_key = set()
@@ -97,16 +101,18 @@ def cacheable(fn):
         except TypeError:
             return fn(*args, **kwargs)
         cache_key = (
+            fn,
             tuple(map(convert_value, args)), frozenset(kwargs_cache_key))
         try:
-            return cache[cache_key]
+            return STRATEGY_CACHE[cache_key]
         except TypeError:
             return fn(*args, **kwargs)
         except KeyError:
             result = fn(*args, **kwargs)
-            cache[cache_key] = result
+            if not isinstance(result, SearchStrategy) or result.is_cacheable:
+                STRATEGY_CACHE[cache_key] = result
             return result
-    cached_strategy.__clear_cache = cache.clear
+    cached_strategy.__clear_cache = STRATEGY_CACHE.clear
     return cached_strategy
 
 
@@ -166,9 +172,13 @@ def nothing():
 
 
 def just(value):
-    """Return a strategy which only generates value.
+    """Return a strategy which only generates ``value``.
 
-    Note: value is not copied. Be wary of using mutable values.
+    Note: ``value`` is not copied. Be wary of using mutable values.
+
+    If ``value`` is the result of a callable, you can use
+    :func:`builds(callable) <hypothesis.strategies.builds>` instead
+    of ``just(callable())`` to get a fresh value each time.
 
     """
     from hypothesis.searchstrategy.misc import JustStrategy
@@ -397,23 +407,33 @@ def tuples(*args):
 
 @defines_strategy
 def sampled_from(elements):
-    """Returns a strategy which generates any value present in the iterable
-    elements.
+    """Returns a strategy which generates any value present in ``elements``.
 
-    Note that as with just, values will not be copied and thus you
-    should be careful of using mutable data.
+    Note that as with :func:`~hypotheses.strategies.just`, values will not be
+    copied and thus you should be careful of using mutable data.
+
+    ``sampled_from`` supports ordered collections, as well as
+    :class:`~python:enum.Enum` objects.  :class:`~python:enum.Flag` objects
+    may also generate any combination of their members.
 
     """
-
-    from hypothesis.searchstrategy.misc import SampledFromStrategy, \
-        JustStrategy
+    from hypothesis.searchstrategy.misc import SampledFromStrategy
     from hypothesis.internal.conjecture.utils import check_sample
-    elements = check_sample(elements)
-    if not elements:
+    values = check_sample(elements)
+    if not values:
         return nothing()
-    if len(elements) == 1:
-        return JustStrategy(elements[0])
-    return SampledFromStrategy(elements)
+    if len(values) == 1:
+        return just(values[0])
+    if hasattr(enum, 'Flag') and isclass(elements) and \
+            issubclass(elements, enum.Flag):
+        # Combinations of enum.Flag members are also members.  We generate
+        # these dynamically, because static allocation takes O(2^n) memory.
+        return sets(sampled_from(values), min_size=1).map(
+            lambda s: reduce(operator.or_, s))
+    return SampledFromStrategy(values)
+
+
+_AVERAGE_LIST_LENGTH = 5.0
 
 
 @cacheable
@@ -923,11 +943,18 @@ def from_type(thing):
     """
     from hypothesis.searchstrategy import types
     if not isinstance(thing, type):
-        # Under Python 3.6, Unions are not instances of `type` - but we still
-        # want to resolve them!  This __origin__ check only passes if thing is
-        # a Union with parameters; if it doesn't we can't resolve it anyway.
         try:
+            # At runtime, `typing.NewType` returns an identity function rather
+            # than an actual type, but we can check that for a possible match
+            # and then read the magic attribute to unwrap it.
             import typing
+            if all([
+                hasattr(thing, '__supertype__'), hasattr(typing, 'NewType'),
+                isfunction(thing), getattr(thing, '__module__', 0) == 'typing'
+            ]):
+                return from_type(thing.__supertype__)
+            # Under Python 3.6, Unions are not instances of `type` - but we
+            # still want to resolve them!
             if getattr(thing, '__origin__', None) is typing.Union:
                 args = sorted(thing.__args__, key=types.type_sorting_key)
                 return one_of([from_type(t) for t in args])
@@ -983,6 +1010,9 @@ def from_type(thing):
             and hasattr(thing, '_field_types'):
         kwargs = {k: from_type(thing._field_types[k]) for k in thing._fields}
         return builds(thing, **kwargs)
+    if issubclass(thing, enum.Enum):
+        assert len(thing), repr(thing) + ' has no members to sample'
+        return sampled_from(thing)
     # If the constructor has an annotation for every required argument,
     # we can (and do) use builds() without supplying additional arguments.
     required = required_args(thing)
@@ -1484,16 +1514,25 @@ def choices():
 
 @cacheable
 @defines_strategy_with_reusable_values
-def uuids():
+def uuids(version=None):
     """Returns a strategy that generates :class:`UUIDs <uuid.UUID>`.
 
+    If the optional version argument is given, value is passed through
+    to :class:`~python:uuid.UUID` and only UUIDs of that version will
+    be generated.
+
     All returned values from this will be unique, so e.g. if you do
     ``lists(uuids())`` the resulting list will never contain duplicates.
 
     """
     from uuid import UUID
+    if version not in (None, 1, 2, 3, 4, 5):
+        raise InvalidArgument((
+            'version=%r, but version must be in (None, 1, 2, 3, 4, 5) '
+            'to pass to the uuid.UUID constructor.') % (version, )
+        )
     return shared(randoms(), key='hypothesis.strategies.uuids.generator').map(
-        lambda r: UUID(int=r.getrandbits(128))
+        lambda r: UUID(version=version, int=r.getrandbits(128))
     )
 
 
@@ -1616,132 +1655,8 @@ def register_type_strategy(custom_type, strategy):
     types._global_type_lookup[custom_type] = strategy
     from_type.__clear_cache()
 
-# Private API below here
-
-
-@check_function
-def check_type(typ, arg, name=''):
-    if name:
-        name += '='
-    if not isinstance(arg, typ):
-        if isinstance(typ, type):
-            typ_string = typ.__name__
-        else:
-            typ_string = 'one of %s' % (
-                ', '.join(t.__name__ for t in typ))
-        raise InvalidArgument('Expected %s but got %s%r (type=%s)'
-                              % (typ_string, name, arg, type(arg).__name__))
-
-
-@check_function
-def check_strategy(arg, name=''):
-    check_type(SearchStrategy, arg, name)
-
-
-@check_function
-def check_valid_integer(value):
-    """Checks that value is either unspecified, or a valid integer.
-
-    Otherwise raises InvalidArgument.
-
-    """
-    if value is None:
-        return
-    check_type(integer_types, value)
-
-
-@check_function
-def check_valid_bound(value, name):
-    """Checks that value is either unspecified, or a valid interval bound.
-
-    Otherwise raises InvalidArgument.
-
-    """
-    if value is None or isinstance(value, integer_types + (Rational,)):
-        return
-    if math.isnan(value):
-        raise InvalidArgument(u'Invalid end point %s=%r' % (name, value))
-
-
-@check_function
-def try_convert(typ, value, name):
-    if value is None:
-        return None
-    if isinstance(value, typ):
-        return value
-    try:
-        return typ(value)
-    except TypeError:
-        raise InvalidArgument(
-            'Cannot convert %s=%r of type %s to type %s' % (
-                name, value, type(value).__name__, typ.__name__
-            )
-        )
-    except ValueError:
-        raise InvalidArgument(
-            'Cannot convert %s=%r to type %s' % (
-                name, value, typ.__name__
-            )
-        )
-
-
-@check_function
-def check_valid_size(value, name):
-    """Checks that value is either unspecified, or a valid non-negative size
-    expressed as an integer/float.
-
-    Otherwise raises InvalidArgument.
-
-    """
-    if value is None:
-        return
-    check_type(integer_types + (float,), value)
-    if value < 0:
-        raise InvalidArgument(u'Invalid size %s %r < 0' % (value, name))
-    if isinstance(value, float) and math.isnan(value):
-        raise InvalidArgument(u'Invalid size %s %r' % (value, name))
-
-
-@check_function
-def check_valid_interval(lower_bound, upper_bound, lower_name, upper_name):
-    """Checks that lower_bound and upper_bound are either unspecified, or they
-    define a valid interval on the number line.
-
-    Otherwise raises InvalidArgument.
-
-    """
-    if lower_bound is None or upper_bound is None:
-        return
-    if upper_bound < lower_bound:
-        raise InvalidArgument(
-            'Cannot have %s=%r < %s=%r' % (
-                upper_name, upper_bound, lower_name, lower_bound
-            ))
-
-
-@check_function
-def check_valid_sizes(min_size, average_size, max_size):
-    check_valid_size(min_size, 'min_size')
-    check_valid_size(max_size, 'max_size')
-    check_valid_size(average_size, 'average_size')
-    check_valid_interval(min_size, max_size, 'min_size', 'max_size')
-    check_valid_interval(average_size, max_size, 'average_size', 'max_size')
-    check_valid_interval(min_size, average_size, 'min_size', 'average_size')
-
-    if average_size is not None:
-        if (
-            (max_size is None or max_size > 0) and
-            average_size is not None and average_size <= 0.0
-        ):
-            raise InvalidArgument(
-                'Cannot have average_size=%r < min_size=%r' % (
-                    average_size, min_size
-                ))
-
-
-_AVERAGE_LIST_LENGTH = 5.0
-
 
+@cacheable
 def deferred(definition):
     """A deferred strategy allows you to write a strategy that references other
     strategies that have not yet been defined. This allows for the easy
